/*
  
Project: OscPocketO
Description: Arduino Pocket Drums using the Mozzi library v2
Author: Staffan Melin, staffan.melin@oscillator.se
License: GNU General Public License v3.0
Version: 20240904
Project site: http://www.oscillator.se/arduino

*/

#define DEBUG


#include <MozziConfigValues.h>
#define MOZZI_ANALOG_READ_RESOLUTION 10
#define MOZZI_AUDIO_PIN_1 TIMER1_B_PIN
#define MOZZI_AUDIO_PIN_1_REGISTER OCR1B
// probably not necessary
#define MOZZI_AUDIO_PIN_2 TIMER1_A_PIN
#define MOZZI_AUDIO_PIN_2_REGISTER OCR1A
#include <Mozzi.h>

#include <ADSR.h>
#include <Oscil.h> // oscillator template
#include <tables/sin2048_int8.h>
#include <tables/triangle2048_int8.h>
#include <tables/whitenoise8192_int8.h>

// LCD
#include <LiquidCrystal.h>

// EEPROM
#include <EEPROM.h>

// use #define for CONTROL_RATE, not a constant
#define MOZZI_CONTROL_RATE 128 // Hz, powers of 2 are most reliable


#define DRUM_LEVEL 150 // 128
#define DRUM_LEVEL_B 200
#define DRUM_NOISE_FREQ 100 // doesn't matter for white noise

#define HIHAT_FREQ1 100
#define HIHAT_FREQ2 128
#define HIHAT_FREQ3 256
#define HIHAT_FREQ4 384
#define HIHAT_FREQ5 512

typedef struct {
  // settings
  int sMFrequency;
  unsigned int sMAttackTime;
  unsigned int sMDecayTime;
  unsigned int sMSustainTime;
  unsigned int sMReleaseTime;
  unsigned int sMReleaseTimeP;
  byte sMAttackLevel;
  byte sMDecayLevel;
  byte sMSustainLevel;
  byte sMReleaseLevel;
//  unsigned int sMFilterFrequency;
//  Q0n8 sMFilterResonance;
  byte sMEnvSlope;
  // state
  byte sMEnvValueA;
  byte sMEnvValueP;
} MDSynth;

MDSynth gMSynthKick;
MDSynth gMSynthSnare;
MDSynth gMSynthHihat;
MDSynth gMSynthClap;
MDSynth gMSynthCrash;
MDSynth gMSynthTomHi;

// oscillators
Oscil <SIN2048_NUM_CELLS, MOZZI_AUDIO_RATE> gMDOscKick(SIN2048_DATA);
Oscil <SIN2048_NUM_CELLS, MOZZI_AUDIO_RATE> gMDOscSnare(SIN2048_DATA);
Oscil <WHITENOISE8192_NUM_CELLS, MOZZI_AUDIO_RATE> gMDOscSnareN(WHITENOISE8192_DATA);
Oscil <WHITENOISE8192_NUM_CELLS, MOZZI_AUDIO_RATE> gMDOscHihatN(WHITENOISE8192_DATA);
Oscil <WHITENOISE8192_NUM_CELLS, MOZZI_AUDIO_RATE> gMDOscClapN(WHITENOISE8192_DATA);
Oscil <WHITENOISE8192_NUM_CELLS, MOZZI_AUDIO_RATE> gMDOscCrashN(WHITENOISE8192_DATA);
Oscil <TRIANGLE2048_NUM_CELLS, MOZZI_AUDIO_RATE> gMDOscTomHi(TRIANGLE2048_DATA);

// envelopes
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvKickA;
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvKickP;
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvSnareA;
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvSnareP;
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvHihatA;
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvClapA;
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvCrashA;
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvTomHiA;
ADSR <CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvTomHiP;



// SEQUENCER

typedef struct {
  uint8_t gSeqGatePercent;
  byte gSeqBPM = 120; // tempo, BPM (beats per minute)
  unsigned int gSeqT16 = 125; // length in ms of 1/16
} SeqBase;

SeqBase gSeqBase;

// index of current note (0-MAX_NOTES)
byte gSeqPatternIndex;
byte gSeqNoteIndex;
byte gSeqNoteEdit; // currently edited note, < MAX_NOTES
byte gSeqParamEdit; // edited parameter for UI_MODE_EDIT_*
unsigned long gSeqTimeCurrent;
unsigned long gSeqTimeLast;
unsigned long gSeqTimeGate;
bool gSeqNoteOn;
bool gSeqPlay; // true if sequence is playing

byte gPlayMode; // play mode: 0 = no sound; 10-13 = note 1-3 is playing

#define MAX_PATTERNS 3
#define MAX_NOTES 16 // max # of notes in sequence
#define D_KICK 1
#define D_SNARE 2
#define D_HIHAT 4
#define D_CLAP 8
#define D_CRASH 16
#define D_TOMHI 32

byte gSeqNotes[MAX_PATTERNS][MAX_NOTES] = 
  {
    {
      D_KICK, 0, D_HIHAT, 0, 
      D_SNARE, 0, D_HIHAT, 0, 
      D_KICK, 0, D_HIHAT, 0, 
      D_SNARE, 0, D_HIHAT, D_KICK
    },
    {
      D_KICK + D_CRASH, 0, D_HIHAT, 0, 
      D_KICK, 0, D_HIHAT, 0, 
      D_KICK, 0, D_HIHAT, 0, 
      D_CLAP, 0, D_HIHAT, D_KICK
    },
    {
      D_KICK, 0, 0, 0, 
      D_KICK, 0, 0, 0, 
      D_KICK, 0, 0, 0, 
      D_KICK, 0, 0, 0
    }
  };



#define PIN_SYNC_IN 11
#define PIN_SYNC_OUT 12
bool gSyncOn = false;
bool gSyncNoteOn = false;
byte gSyncMode;
byte gSyncCount;
#define SYNC_NONE 0 // use internal sync, do not send out sync
#define SYNC_INT 1 // use internal sync, send out sync
#define SYNC_EXT 2 // use external sync, send out sync
#define SYNC_EXT24 3 // use external sync, 24ppq



// UI

#define PIN_LED 13

// LCD
#define PIN_LCD_D4 4
#define PIN_LCD_D5 5
#define PIN_LCD_D6 6
#define PIN_LCD_D7 7
#define PIN_LCD_RS 8
#define PIN_LCD_EN 9

// buttons, all
#define PIN_BUTTONS A0

// LCD
LiquidCrystal gUILCD(PIN_LCD_RS, PIN_LCD_EN, PIN_LCD_D4, PIN_LCD_D5, PIN_LCD_D6, PIN_LCD_D7);

enum UIMode { UI_MODE_START, UI_MODE_SYNC, 
              UI_MODE_PATTERN, UI_MODE_EDIT, 
              UI_MODE_TEMPO, UI_MODE_GATE,
              UI_MODE_EDIT_KICK, UI_MODE_EDIT_SNARE, UI_MODE_EDIT_HIHAT, UI_MODE_EDIT_CLAP, UI_MODE_EDIT_CRASH, UI_MODE_EDIT_TOMHI,
              UI_MODE_PLAY, UI_MODE_TOOLS};

UIMode gUIMode = UI_MODE_START;
byte gUIModeTools = 0;

enum UIButton {UI_BUTTON_NONE, UI_BUTTON_SELECT, UI_BUTTON_LEFT, UI_BUTTON_RIGHT, UI_BUTTON_UP, UI_BUTTON_DOWN};



// UTILITY

void setTempo(void) {
  gSeqBase.gSeqT16 = 1000 / ((gSeqBase.gSeqBPM * 4) / ((float)60));
}

void calcGate() {
  gSeqTimeGate = (gSeqBase.gSeqT16 * gSeqBase.gSeqGatePercent) / 100;        
}

void playDKick()
{
  gMDOscKick.setFreq(gMSynthKick.sMFrequency);
  gMEnvKickA.noteOn();
  gMEnvKickP.noteOn();
  gMEnvKickA.noteOff();
  gMEnvKickP.noteOff();
}

void playDSnare()
{
  gMDOscSnare.setFreq(gMSynthSnare.sMFrequency);
  gMEnvSnareA.noteOn();
  gMEnvSnareP.noteOn();
  gMEnvSnareA.noteOff();
  gMEnvSnareP.noteOff();
}

void playDHihat()
{
  gMEnvHihatA.setReleaseTime(gMSynthHihat.sMReleaseTime);
  gMEnvHihatA.noteOn();
  gMEnvHihatA.noteOff();
}

void playDHihatO()
{
  gMEnvHihatA.setReleaseTime(gMSynthHihat.sMReleaseTime >> 1);
  gMEnvHihatA.noteOn();
  gMEnvHihatA.noteOff();
}

void playDClap()
{
  gMEnvClapA.noteOn();
  gMEnvClapA.noteOff();
}

void playDCrash()
{
  gMEnvCrashA.noteOn();
  gMEnvCrashA.noteOff();
}

void playDTomHi()
{
  gMDOscTomHi.setFreq(gMSynthTomHi.sMFrequency);
  gMEnvTomHiA.noteOn();
  gMEnvTomHiP.noteOn();
  gMEnvTomHiA.noteOff();
  gMEnvTomHiP.noteOff();
}

void playDTomLo()
{
  gMDOscTomHi.setFreq(gMSynthTomHi.sMFrequency << 1);
  gMEnvTomHiA.noteOn();
  gMEnvTomHiP.noteOn();
  gMEnvTomHiA.noteOff();
  gMEnvTomHiP.noteOff();
}

void playNote() {
  byte aNote = gSeqNotes[gSeqPatternIndex][gSeqNoteIndex];

  // 01 - kick
  if (aNote & D_KICK) {
    playDKick();
  }
  
  // 02 - snare
  if (aNote & D_SNARE) {
    playDSnare();
  }
  
  // 04 - hihat
  if (aNote & D_HIHAT) {
    playDHihat();
  }

  // 08 - clap
  if (aNote & D_CLAP) {
    playDClap();
  }

  // 16 - crash
  if (aNote & D_CRASH) {
    playDCrash();
  }

  // 32 - tom hi
  if (aNote & D_TOMHI) {
    playDTomHi();
  }

}



// EEPROM size: 1024 bytes
void saveToEEPROM() {
  int aAddress = 0;
/*
  // save synth settings
  EEPROM.put(aAddress, gMSynth);
  aAddress += sizeof(MSynth);
*/
  // save sequencer settings
  EEPROM.put(aAddress, gSeqBase);
  aAddress += sizeof(SeqBase);

  // save note data
  for (byte p = 0; p < MAX_PATTERNS; p++) {
    for (byte n = 0; n < MAX_NOTES; n++) {
      EEPROM.write(aAddress++, gSeqNotes[p][n]);
    }
  }
}



void loadFromEEPROM() {
  int aAddress = 0;
/*
  // load synth settings
  EEPROM.get(aAddress, gMSynth);
  aAddress += sizeof(MSynth);
*/

  // load sequencer settings
  EEPROM.get(aAddress, gSeqBase);
  aAddress += sizeof(SeqBase);

  // load note data  
  for (byte p = 0; p < MAX_PATTERNS; p++) {
    for (byte n = 0; n < MAX_NOTES; n++) {
      gSeqNotes[p][n] = EEPROM.read(aAddress++);
    }
  }
}



// UI

void UIDraw() {
  
  gUILCD.clear();
  
  switch (gUIMode) {
    
  case UI_MODE_START:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: START DRUMS");
    gUILCD.setCursor(0, 1);
    if (gSeqPlay) {
      gUILCD.print ("Play");
    } else {
      gUILCD.print ("Stop");
    } 
    break;

  case UI_MODE_SYNC:
    gUILCD.setCursor(0, 0);
    gUILCD.print("M: SYNC");
    gUILCD.setCursor(0, 1);
    switch (gSyncMode) {
      case SYNC_NONE:
        gUILCD.print("None");
        break;
      case SYNC_INT:
        gUILCD.print("Internal");
        break;
      case SYNC_EXT:
        gUILCD.print("External");
        break;
      case SYNC_EXT24:
        gUILCD.print("Ext24");
        break;
    }
    break;

  case UI_MODE_PATTERN:
    gUILCD.setCursor(0, 0);
    gUILCD.print("M: PATTERN");
    gUILCD.setCursor(0, 1);
    gUILCD.write('0' + gSeqPatternIndex);
    break;

  case UI_MODE_EDIT:
    gUILCD.setCursor(0, 0);
    for (byte i = 0; i < MAX_NOTES; i++) {
      byte aValue = gSeqNotes[gSeqPatternIndex][i];
      byte aTen = aValue / 10;
      gUILCD.setCursor(i, 0);
      gUILCD.write('0' + aTen);
      gUILCD.setCursor(i, 1);
      gUILCD.write('0' +  aValue - aTen * 10);
    }
    gUILCD.setCursor(gSeqNoteEdit, 1);
    break;

  case UI_MODE_TEMPO:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: TEMPO");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gSeqBase.gSeqBPM);
    break;

  case UI_MODE_GATE:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: GATE");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gSeqBase.gSeqGatePercent);
    break;
    
  case UI_MODE_EDIT_KICK:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: EDIT KICK");
    gUILCD.setCursor(0, 1);
    // f:999 t:999 e:9
    gUILCD.print("f:    t:    e: ");
    gUILCD.setCursor(2, 1);
    gUILCD.print(gMSynthKick.sMFrequency);
    gUILCD.setCursor(8, 1);
    gUILCD.print(gMSynthKick.sMReleaseTime);
    gUILCD.setCursor(14, 1);
    gUILCD.print(gMSynthKick.sMEnvSlope);
    switch (gSeqParamEdit) {
      case 0:
        gUILCD.setCursor(2,1);
        break;
      case 1:
        gUILCD.setCursor(8,1);
        break;
      case 2:
        gUILCD.setCursor(14,1);
        break;
    }
    break;

  case UI_MODE_EDIT_SNARE:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: EDIT SNARE");
    gUILCD.setCursor(0, 1);
    // f:999 t:999 e:9
    gUILCD.print("f:    t:    e: ");
    gUILCD.setCursor(2, 1);
    gUILCD.print(gMSynthSnare.sMFrequency);
    gUILCD.setCursor(8, 1);
    gUILCD.print(gMSynthSnare.sMReleaseTime);
    gUILCD.setCursor(14, 1);
    gUILCD.print(gMSynthSnare.sMEnvSlope);
    switch (gSeqParamEdit) {
      case 0:
        gUILCD.setCursor(2,1);
        break;
      case 1:
        gUILCD.setCursor(8,1);
        break;
      case 2:
        gUILCD.setCursor(14,1);
        break;
    }
    break;

  case UI_MODE_EDIT_HIHAT:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: EDIT HIHAT");
    gUILCD.setCursor(0, 1);
    // f:999 t:999
    gUILCD.print("f:    t:   ");
    gUILCD.setCursor(2, 1);
    gUILCD.print(gMSynthHihat.sMFrequency);
    gUILCD.setCursor(8, 1);
    gUILCD.print(gMSynthHihat.sMReleaseTime);
    switch (gSeqParamEdit) {
      case 0:
        gUILCD.setCursor(2,1);
        break;
      case 1:
        gUILCD.setCursor(8,1);
        break;
    }
    break;

  case UI_MODE_EDIT_CLAP:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: EDIT CLAP");
    gUILCD.setCursor(0, 1);
    // t:999
    gUILCD.print("t:    ");
    gUILCD.setCursor(2, 1);
    gUILCD.print(gMSynthClap.sMReleaseTime);
    gUILCD.setCursor(2, 1);
    break;

  case UI_MODE_EDIT_CRASH:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: EDIT CRASH");
    gUILCD.setCursor(0, 1);
    // t:9999
    gUILCD.print("t:     ");
    gUILCD.setCursor(2, 1);
    gUILCD.print(gMSynthCrash.sMReleaseTime);
    gUILCD.setCursor(2,1);
    break;
    
  case UI_MODE_EDIT_TOMHI:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: EDIT TOMHI");
    gUILCD.setCursor(0, 1);
    // f:999 t:999 e:9
    gUILCD.print("f:    t:    e: ");
    gUILCD.setCursor(2, 1);
    gUILCD.print(gMSynthTomHi.sMFrequency);
    gUILCD.setCursor(8, 1);
    gUILCD.print(gMSynthTomHi.sMReleaseTime);
    gUILCD.setCursor(14, 1);
    gUILCD.print(gMSynthTomHi.sMEnvSlope);
    switch (gSeqParamEdit) {
      case 0:
        gUILCD.setCursor(2,1);
        break;
      case 1:
        gUILCD.setCursor(8,1);
        break;
      case 2:
        gUILCD.setCursor(14,1);
        break;
    }
    break;

  case UI_MODE_PLAY:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: PLAY");
    gUILCD.setCursor(0, 1);
    gUILCD.print("Kck Snr Tom Cra");
    break;

  case UI_MODE_TOOLS:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: TOOLS");
    gUILCD.setCursor(0, 1);
    gUILCD.print("SLRBC");
    gUILCD.setCursor(gUIModeTools, 1);
    break;

  }

}

void UIHandle() {
  int aUIButtonValue;
  UIButton aUIButton;

  static unsigned long aUIButtonPreviousTime = millis();

  aUIButtonValue = mozziAnalogRead(PIN_BUTTONS);

  #ifdef DEBUG
    // Serial.println(aUIButtonValue);
  #endif

  // is any button pressed?
  if (aUIButtonValue < 900)
  {
    // check which one
    if (aUIButtonValue < 50)
    {
      aUIButton = UI_BUTTON_RIGHT;
    } else if (aUIButtonValue < 150) {
      aUIButton = UI_BUTTON_UP;
    } else if (aUIButtonValue < 300) {
      aUIButton = UI_BUTTON_DOWN;
    } else if (aUIButtonValue < 500) {
      aUIButton = UI_BUTTON_LEFT;
    } else if (aUIButtonValue < 700) {
      aUIButton = UI_BUTTON_SELECT;
    } else {
      aUIButton = UI_BUTTON_NONE;
    }

    // rightmost button is RST = reset = resets Arduino

    // debounce buttons
    if (aUIButton != UI_BUTTON_NONE && (millis() - aUIButtonPreviousTime > 200)) {
      aUIButtonPreviousTime = millis();
      
      switch (gUIMode) {
        
      case UI_MODE_START:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_SYNC;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (!gSeqPlay) {
            gSeqNoteIndex = 0;
            gSeqTimeCurrent = millis();
            gSeqTimeLast = gSeqTimeCurrent;
            gSeqNoteOn = false;
            //playNote();
            gSeqPlay = true;
            gSyncNoteOn = false;
            gSyncOn = false;
            gSyncCount = 0;

          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqPlay) {
            gSeqPlay = false;
            // turn off envelopes?
            digitalWrite(PIN_LED, LOW);            
            gSeqNoteOn = false;
          }
        } 
        break;
        
      case UI_MODE_SYNC:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_PATTERN;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSyncMode < SYNC_EXT24)
          {
            gSyncMode++;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSyncMode > 0)
          {
            gSyncMode--;
            gSeqTimeLast = gSeqTimeCurrent; // going back to SYNC_INT (or SYNC_NONE), make sure gSeqTimeLast is updated
          }
        }
        break;

      case UI_MODE_PATTERN:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_EDIT;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqPatternIndex < (MAX_PATTERNS - 1))
          {
            gSeqPatternIndex++;
          } else {
            gSeqPatternIndex = 0;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqPatternIndex > 0)
          {
            gSeqPatternIndex--;
          } else {
            gSeqPatternIndex = (MAX_PATTERNS - 1);            
          }
        }
        break;

      case UI_MODE_EDIT:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_TEMPO;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqNotes[gSeqPatternIndex][gSeqNoteEdit] < 63)
          {
            gSeqNotes[gSeqPatternIndex][gSeqNoteEdit]++;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqNotes[gSeqPatternIndex][gSeqNoteEdit] > 0)
          {
            gSeqNotes[gSeqPatternIndex][gSeqNoteEdit]--;
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gSeqNoteEdit > 0) {
            gSeqNoteEdit--;
          }
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gSeqNoteEdit < MAX_NOTES - 1) {
            gSeqNoteEdit++;
          }
        }
        break;

      case UI_MODE_TEMPO:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_GATE;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqBase.gSeqBPM < 999)
          {
            gSeqBase.gSeqBPM++;
            setTempo();
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqBase.gSeqBPM > 10)
          {
            gSeqBase.gSeqBPM--;
            setTempo();
          }
        }
        break;

      case UI_MODE_GATE:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_EDIT_KICK;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqBase.gSeqGatePercent < 99)
          {
            gSeqBase.gSeqGatePercent++;
            calcGate();
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqBase.gSeqGatePercent > 0)
          {
            gSeqBase.gSeqGatePercent--;
            calcGate();
          }
        }

        break;

      case UI_MODE_EDIT_KICK:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_EDIT_SNARE;
          gSeqParamEdit = 0;
        } else if (aUIButton == UI_BUTTON_UP) {
          switch (gSeqParamEdit) {
            case 0: // freq
              if (gMSynthKick.sMFrequency < 120) {
                gMSynthKick.sMFrequency += 10;
                gMDOscKick.setFreq(gMSynthKick.sMFrequency);
              }
              break;
            case 1: // RTime
              if (gMSynthKick.sMReleaseTime < 700) {
                gMSynthKick.sMReleaseTime += 10;
                gMEnvKickA.setReleaseTime(gMSynthKick.sMReleaseTime);
                gMEnvKickP.setReleaseTime(gMSynthKick.sMReleaseTime);
              }
              break;
            case 2: // env slope
              if (gMSynthKick.sMEnvSlope < 2) {
                gMSynthKick.sMEnvSlope++;
              }
              break;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          switch (gSeqParamEdit) {
            case 0: // freq
              if (gMSynthKick.sMFrequency > 10) {
                gMSynthKick.sMFrequency -= 10;
                gMDOscKick.setFreq(gMSynthKick.sMFrequency);
              }
              break;
            case 1: // RTime
              if (gMSynthKick.sMReleaseTime > 10) {
                gMSynthKick.sMReleaseTime -= 10;
                gMEnvKickA.setReleaseTime(gMSynthKick.sMReleaseTime);
                gMEnvKickP.setReleaseTime(gMSynthKick.sMReleaseTime);
              }
              break;
            case 2: // env slope
              if (gMSynthKick.sMEnvSlope > 0) {
                gMSynthKick.sMEnvSlope--;
              }
              break;
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gSeqParamEdit > 0) {
            gSeqParamEdit--;
          }
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gSeqParamEdit < 2) {
            gSeqParamEdit++;
          }
        }
        break;

      case UI_MODE_EDIT_SNARE:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_EDIT_HIHAT;
          gSeqParamEdit = 0;
        } else if (aUIButton == UI_BUTTON_UP) {
          switch (gSeqParamEdit) {
            case 0: // freq 20-100
              if (gMSynthSnare.sMFrequency < 400) {
                gMSynthSnare.sMFrequency += 10;
                gMDOscSnare.setFreq(gMSynthSnare.sMFrequency);
              }
              break;
            case 1: // RTime 100-500
              if (gMSynthSnare.sMReleaseTime < 500) {
                gMSynthSnare.sMReleaseTime += 10;
                gMEnvSnareA.setReleaseTime(gMSynthSnare.sMReleaseTime);
                gMEnvSnareP.setReleaseTime(gMSynthSnare.sMReleaseTime);
              }
              break;
            case 2: // env slope, 1-3
              if (gMSynthSnare.sMEnvSlope < 3) {
                gMSynthSnare.sMEnvSlope++;
              }
              break;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          switch (gSeqParamEdit) {
            case 0: // freq
              if (gMSynthSnare.sMFrequency > 19) {
                gMSynthSnare.sMFrequency -= 10;
                gMDOscSnare.setFreq(gMSynthSnare.sMFrequency);
              }
              break;
            case 1: // RTime
              if (gMSynthSnare.sMReleaseTime > 50) {
                gMSynthSnare.sMReleaseTime -= 10;
                gMEnvSnareA.setReleaseTime(gMSynthSnare.sMReleaseTime);
                gMEnvSnareP.setReleaseTime(gMSynthSnare.sMReleaseTime);
              }
              break;
            case 2: // env slope
              if (gMSynthSnare.sMEnvSlope > 0) {
                gMSynthSnare.sMEnvSlope--;
              }
              break;
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gSeqParamEdit > 0) {
            gSeqParamEdit--;
          }
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gSeqParamEdit < 2) {
            gSeqParamEdit++;
          }
        }
        break;

      case UI_MODE_EDIT_HIHAT:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_EDIT_CLAP;
          gSeqParamEdit = 0;
        } else if (aUIButton == UI_BUTTON_UP) {
          switch (gSeqParamEdit) {
            case 0: // freq
              if (gMSynthHihat.sMFrequency == HIHAT_FREQ1) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ2;
              } else if (gMSynthHihat.sMFrequency == HIHAT_FREQ2) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ3;
              } else if (gMSynthHihat.sMFrequency == HIHAT_FREQ3) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ4;
              } else if (gMSynthHihat.sMFrequency == HIHAT_FREQ4) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ5;
              } else if (gMSynthHihat.sMFrequency == HIHAT_FREQ5) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ1;
              }
              gMDOscHihatN.setFreq(gMSynthHihat.sMFrequency);
              break;
            case 1: // RTime 100-500
              if (gMSynthHihat.sMReleaseTime < 500) {
                gMSynthHihat.sMReleaseTime += 10;
                gMEnvHihatA.setReleaseTime(gMSynthHihat.sMReleaseTime);
              }
              break;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          switch (gSeqParamEdit) {
            case 0: // freq
              if (gMSynthHihat.sMFrequency == HIHAT_FREQ5) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ4;
              } else if (gMSynthHihat.sMFrequency == HIHAT_FREQ4) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ3;
              } else if (gMSynthHihat.sMFrequency == HIHAT_FREQ3) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ2;
              } else if (gMSynthHihat.sMFrequency == HIHAT_FREQ2) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ1;
              } else if (gMSynthHihat.sMFrequency == HIHAT_FREQ1) {
                gMSynthHihat.sMFrequency = HIHAT_FREQ5;
              }
              gMDOscHihatN.setFreq(gMSynthHihat.sMFrequency);
              break;
            case 1: // RTime
              if (gMSynthHihat.sMReleaseTime > 20) {
                gMSynthHihat.sMReleaseTime -= 10;
                gMEnvHihatA.setReleaseTime(gMSynthHihat.sMReleaseTime);
              }
              break;
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gSeqParamEdit > 0) {
            gSeqParamEdit--;
          }
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gSeqParamEdit < 1) {
            gSeqParamEdit++;
          }
        }

        break;

      case UI_MODE_EDIT_CLAP:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_EDIT_CRASH;
          gSeqParamEdit = 0;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynthClap.sMReleaseTime < 500) {
            gMSynthClap.sMReleaseTime += 10;
            gMEnvClapA.setReleaseTime(gMSynthClap.sMReleaseTime);
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynthClap.sMReleaseTime > 30) {
            gMSynthClap.sMReleaseTime -= 10;
            gMEnvClapA.setReleaseTime(gMSynthClap.sMReleaseTime);
          }
        }
        break;

      case UI_MODE_EDIT_CRASH:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_EDIT_TOMHI;
          gSeqParamEdit = 0;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynthCrash.sMReleaseTime < 4000) {
            gMSynthCrash.sMReleaseTime += 100;
            gMEnvCrashA.setReleaseTime(gMSynthCrash.sMReleaseTime);
          }
          break;
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynthCrash.sMReleaseTime > 300) {
            gMSynthCrash.sMReleaseTime -= 100;
            gMEnvCrashA.setReleaseTime(gMSynthCrash.sMReleaseTime);
          }
          break;
        }
        break;

      case UI_MODE_EDIT_TOMHI:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_PLAY;
          gSeqParamEdit = 0;
        } else if (aUIButton == UI_BUTTON_UP) {
          switch (gSeqParamEdit) {
            case 0: // freq
              if (gMSynthTomHi.sMFrequency < 2000) {
                gMSynthTomHi.sMFrequency += 10;
                gMDOscTomHi.setFreq(gMSynthTomHi.sMFrequency);
              }
              break;
            case 1: // RTime
              if (gMSynthTomHi.sMReleaseTime < 1000) {
                gMSynthTomHi.sMReleaseTime += 10;
                gMEnvTomHiA.setReleaseTime(gMSynthTomHi.sMReleaseTime);
                gMEnvTomHiP.setReleaseTime(gMSynthTomHi.sMReleaseTime);
              }
              break;
            case 2: // env slope, 1-3
              if (gMSynthTomHi.sMEnvSlope < 3) {
                gMSynthTomHi.sMEnvSlope++;
              }
              break;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          switch (gSeqParamEdit) {
            case 0: // freq
              if (gMSynthTomHi.sMFrequency > 19) {
                gMSynthTomHi.sMFrequency -= 10;
                gMDOscTomHi.setFreq(gMSynthTomHi.sMFrequency);
              }
              break;
            case 1: // RTime
              if (gMSynthTomHi.sMReleaseTime > 50) {
                gMSynthTomHi.sMReleaseTime -= 10;
                gMEnvTomHiA.setReleaseTime(gMSynthTomHi.sMReleaseTime);
                gMEnvTomHiP.setReleaseTime(gMSynthTomHi.sMReleaseTime);
              }
              break;
            case 2: // env slope
              if (gMSynthTomHi.sMEnvSlope > 0) {
                gMSynthTomHi.sMEnvSlope--;
              }
              break;
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gSeqParamEdit > 0) {
            gSeqParamEdit--;
          }
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gSeqParamEdit < 2) {
            gSeqParamEdit++;
          }
        }
        break;

      case UI_MODE_PLAY:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_TOOLS;
          gPlayMode = 0;
        } else if (aUIButton == UI_BUTTON_LEFT) {
          playDKick();
          gPlayMode = 10;
        } else if (aUIButton == UI_BUTTON_UP) {
          playDSnare();
          gPlayMode = 11;
        } else if (aUIButton == UI_BUTTON_DOWN) {
//          playDClap();
          playDTomHi();
          gPlayMode = 12;
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          playDCrash();
          gPlayMode = 13;
        }
        break;

      case UI_MODE_TOOLS:
        // Save/Load/Random/Bassline/Copytopatternover
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_START;
          // because the LCD messes up from time to time
          gUILCD.begin(16, 2);
          gUILCD.cursor();
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gUIModeTools < 4)
          {
            gUIModeTools++;
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gUIModeTools > 0)
          {
            gUIModeTools--;
          }
        } else if (aUIButton == UI_BUTTON_UP) {
          switch (gUIModeTools) {
            case 0: // Save
              // stop playing (because this takes time)
              gSeqPlay = false;
              // turn off envelopes?
              digitalWrite(PIN_LED, LOW);            
              gSeqNoteOn = false;
              
              saveToEEPROM();
              break;
            case 1: // Load
              // stop playing (because this takes time)
              gSeqPlay = false;
              // turn off envelopes?
              digitalWrite(PIN_LED, LOW);            
              gSeqNoteOn = false;

              loadFromEEPROM();
              break;
            case 2: // Random
              for (byte n = 0; n < MAX_NOTES; n++) {
                byte aNote = 0;
                if (rand()%100 < 13) {
                  aNote += D_KICK; 
                }
                if (rand()%100 < 13) {
                  aNote += D_SNARE; 
                }
                if (rand()%100 < 13) {
                  aNote += D_HIHAT; 
                }
                if (rand()%100 < 8) {
                  aNote += D_CLAP; 
                }
                if (rand()%100 < 8) {
                  aNote += D_CRASH; 
                }
                if (rand()%100 < 8) {
                  aNote += D_TOMHI; 
                }
                gSeqNotes[gSeqPatternIndex][n] = aNote;
              }

              break;
            case 3: // Bassline
              {
                byte aNote = gSeqNotes[gSeqPatternIndex][gSeqNoteEdit];
                for (byte n = 0; n < MAX_NOTES; n++) {
                  gSeqNotes[gSeqPatternIndex][n] = aNote;
                }
                break;
              }
            case 4: // Copy
              {
                // assume MAX_PATTERNS > 1
                // copy to next larger pattern
                byte aTo = gSeqPatternIndex + 1;
                if (aTo == MAX_PATTERNS) {
                  aTo = 0;
                }
                for (byte n = 0; n < MAX_NOTES; n++) {
                  gSeqNotes[aTo][n] = gSeqNotes[gSeqPatternIndex][n];
                }
                break;
              }
          }
        }
        break;

      } // switch (gUIMode)
      
      UIDraw();
      
    } // debounce
  }  // if (gUIButton < 900)
}



// SETUP

void setup()
{
  #ifdef DEBUG
    Serial.begin(115200);
  #endif

  setTempo();

  // Arduino
  
  pinMode(PIN_LED, OUTPUT);
  pinMode(PIN_SYNC_IN, INPUT_PULLUP);
  pinMode(PIN_SYNC_OUT, OUTPUT);
  
  gUILCD.begin(16, 2);
  gUILCD.cursor();
  gUILCD.clear();

  // MOZZI
  
  gMSynthKick.sMFrequency = 45; // (setting)
  gMSynthKick.sMAttackTime = 0;
  gMSynthKick.sMDecayTime = 0;
  gMSynthKick.sMSustainTime = 0;
  gMSynthKick.sMReleaseTime = 170;
  gMSynthKick.sMReleaseTimeP = 170;
  gMSynthKick.sMAttackLevel = DRUM_LEVEL_B;
  gMSynthKick.sMDecayLevel = DRUM_LEVEL_B;
  gMSynthKick.sMSustainLevel = DRUM_LEVEL_B;
  gMSynthKick.sMReleaseLevel = 0;
//  gMSynthKick.sMFilterFrequency = 0;
//  gMSynthKick.sMFilterResonance = 0;
  gMSynthKick.sMEnvSlope = 1;

  gMEnvKickA.setLevels(gMSynthKick.sMAttackLevel, gMSynthKick.sMDecayLevel, gMSynthKick.sMSustainLevel, gMSynthKick.sMReleaseLevel);
  gMEnvKickA.setTimes(gMSynthKick.sMAttackTime, gMSynthKick.sMDecayTime, gMSynthKick.sMSustainTime, gMSynthKick.sMReleaseTime);
  gMEnvKickP.setLevels(gMSynthKick.sMAttackLevel, gMSynthKick.sMDecayLevel, gMSynthKick.sMSustainLevel, gMSynthKick.sMReleaseLevel);
  gMEnvKickP.setTimes(gMSynthKick.sMAttackTime, gMSynthKick.sMDecayTime, gMSynthKick.sMSustainTime, gMSynthKick.sMReleaseTimeP);

  gMSynthSnare.sMFrequency = 150; // (setting)
  gMSynthSnare.sMAttackTime = 0;
  gMSynthSnare.sMDecayTime = 0;
  gMSynthSnare.sMSustainTime = 0;
  gMSynthSnare.sMReleaseTime = 160; // (setting) 140-300
  gMSynthSnare.sMReleaseTimeP = 140;
  gMSynthSnare.sMAttackLevel = DRUM_LEVEL;
  gMSynthSnare.sMDecayLevel = DRUM_LEVEL;
  gMSynthSnare.sMSustainLevel = DRUM_LEVEL;
  gMSynthSnare.sMReleaseLevel = 0;
//  gMSynthSnare.sMFilterFrequency = 0;
//  gMSynthSnare.sMFilterResonance = 0;
  gMSynthSnare.sMEnvSlope = 1;

  gMEnvSnareA.setLevels(gMSynthSnare.sMAttackLevel, gMSynthSnare.sMDecayLevel, gMSynthSnare.sMSustainLevel, gMSynthSnare.sMReleaseLevel);
  gMEnvSnareA.setTimes(gMSynthSnare.sMAttackTime, gMSynthSnare.sMDecayTime, gMSynthSnare.sMSustainTime, gMSynthSnare.sMReleaseTime);
  gMEnvSnareP.setLevels(gMSynthSnare.sMAttackLevel, gMSynthSnare.sMDecayLevel, gMSynthSnare.sMSustainLevel, gMSynthSnare.sMReleaseLevel);
  gMEnvSnareP.setTimes(gMSynthSnare.sMAttackTime, gMSynthSnare.sMDecayTime, gMSynthSnare.sMSustainTime, gMSynthSnare.sMReleaseTimeP);

  gMDOscSnareN.setFreq(DRUM_NOISE_FREQ);

  gMSynthClap.sMFrequency = 0; // (only noise)
  gMSynthClap.sMAttackTime = 0;
  gMSynthClap.sMDecayTime = 0;
  gMSynthClap.sMSustainTime = 0;
  gMSynthClap.sMReleaseTime = 110;
  gMSynthClap.sMReleaseTimeP = 0;
  gMSynthClap.sMAttackLevel = DRUM_LEVEL;
  gMSynthClap.sMDecayLevel = DRUM_LEVEL;
  gMSynthClap.sMSustainLevel = DRUM_LEVEL;
  gMSynthClap.sMReleaseLevel = 0;
//  gMSynthClap.sMFilterFrequency = 1200; // setting
//  gMSynthClap.sMFilterResonance = 120;
  gMSynthClap.sMEnvSlope = 0;

  gMEnvClapA.setLevels(gMSynthClap.sMAttackLevel, gMSynthClap.sMDecayLevel, gMSynthClap.sMSustainLevel, gMSynthClap.sMReleaseLevel);
  gMEnvClapA.setTimes(gMSynthClap.sMAttackTime, gMSynthClap.sMDecayTime, gMSynthClap.sMSustainTime, gMSynthClap.sMReleaseTime);

  gMDOscClapN.setFreq(DRUM_NOISE_FREQ);

  gMSynthHihat.sMFrequency = HIHAT_FREQ1; // (only noise)
  gMSynthHihat.sMAttackTime = 0;
  gMSynthHihat.sMDecayTime = 0;
  gMSynthHihat.sMSustainTime = 0;
  gMSynthHihat.sMReleaseTime = 35; // (settings)
  gMSynthHihat.sMReleaseTimeP = 0;
  gMSynthHihat.sMAttackLevel = DRUM_LEVEL;
  gMSynthHihat.sMDecayLevel = DRUM_LEVEL;
  gMSynthHihat.sMSustainLevel = DRUM_LEVEL;
  gMSynthHihat.sMReleaseLevel = 0;
//  gMSynthHihat.sMFilterFrequency = 5000;
//  gMSynthHihat.sMFilterResonance = 60;
  gMSynthHihat.sMEnvSlope = 0;

  gMEnvHihatA.setLevels(gMSynthHihat.sMAttackLevel, gMSynthHihat.sMDecayLevel, gMSynthHihat.sMSustainLevel, gMSynthHihat.sMReleaseLevel);
  gMEnvHihatA.setTimes(gMSynthHihat.sMAttackTime, gMSynthHihat.sMDecayTime, gMSynthHihat.sMSustainTime, gMSynthHihat.sMReleaseTime);

  gMDOscHihatN.setFreq(gMSynthHihat.sMFrequency);

  gMSynthCrash.sMFrequency = 0; // (only noise)
  gMSynthCrash.sMAttackTime = 0;
  gMSynthCrash.sMDecayTime = 0;
  gMSynthCrash.sMSustainTime = 0;
  gMSynthCrash.sMReleaseTime = 500; // (setting)
  gMSynthCrash.sMReleaseTimeP = 0;
  gMSynthCrash.sMAttackLevel = DRUM_LEVEL;
  gMSynthCrash.sMDecayLevel = DRUM_LEVEL;
  gMSynthCrash.sMSustainLevel = DRUM_LEVEL;
  gMSynthCrash.sMReleaseLevel = 0;
  gMSynthCrash.sMEnvSlope = 0;

  gMEnvCrashA.setLevels(gMSynthCrash.sMAttackLevel, gMSynthCrash.sMDecayLevel, gMSynthCrash.sMSustainLevel, gMSynthCrash.sMReleaseLevel);
  gMEnvCrashA.setTimes(gMSynthCrash.sMAttackTime, gMSynthCrash.sMDecayTime, gMSynthCrash.sMSustainTime, gMSynthCrash.sMReleaseTime);

  gMDOscCrashN.setFreq(DRUM_NOISE_FREQ);


  gMSynthTomHi.sMFrequency = 100; // (setting)
  gMSynthTomHi.sMAttackTime = 0;
  gMSynthTomHi.sMDecayTime = 0;
  gMSynthTomHi.sMSustainTime = 0;
  gMSynthTomHi.sMReleaseTime = 200;
  gMSynthTomHi.sMReleaseTimeP = 200;
  gMSynthTomHi.sMAttackLevel = DRUM_LEVEL;
  gMSynthTomHi.sMDecayLevel = DRUM_LEVEL;
  gMSynthTomHi.sMSustainLevel = DRUM_LEVEL;
  gMSynthTomHi.sMReleaseLevel = 0;
//  gMSynthTomHi.sMFilterFrequency = 0;
//  gMSynthTomHi.sMFilterResonance = 0;
  gMSynthTomHi.sMEnvSlope = 1;

  gMEnvTomHiA.setLevels(gMSynthTomHi.sMAttackLevel, gMSynthTomHi.sMDecayLevel, gMSynthTomHi.sMSustainLevel, gMSynthTomHi.sMReleaseLevel);
  gMEnvTomHiA.setTimes(gMSynthTomHi.sMAttackTime, gMSynthTomHi.sMDecayTime, gMSynthTomHi.sMSustainTime, gMSynthTomHi.sMReleaseTime);
  gMEnvTomHiP.setLevels(gMSynthTomHi.sMAttackLevel, gMSynthTomHi.sMDecayLevel, gMSynthTomHi.sMSustainLevel, gMSynthTomHi.sMReleaseLevel);
  gMEnvTomHiP.setTimes(gMSynthTomHi.sMAttackTime, gMSynthTomHi.sMDecayTime, gMSynthTomHi.sMSustainTime, gMSynthTomHi.sMReleaseTimeP);

  // SEQUENCER
  
  // note lengths
  // see setTempo() function
  gSeqBase.gSeqBPM = 120; // tempo, BPM (beats per minute)
  gSeqBase.gSeqT16 = 125; // length in ms of 1/16
  gSeqBase.gSeqGatePercent = 50;
  
  gSeqPatternIndex = 0;
  gSeqNoteIndex = 0;
  gSeqNoteEdit = 0; // currently edited note, < MAX_NOTES
  gSeqTimeCurrent = millis();
  gSeqTimeLast = gSeqTimeCurrent;
  gSeqTimeGate = (gSeqBase.gSeqT16 >> 1);
  gSeqNoteOn = false;
  gSeqPlay = false;

  gSyncMode = SYNC_NONE;
  gSyncCount = 0;

  UIDraw();

  startMozzi(CONTROL_RATE);

}



// MOZZI: put changing controls in here

void updateControl()
{

  if (gSeqPlay) {

    gSeqTimeCurrent = millis();

    if ((gSyncMode == SYNC_EXT) || (gSyncMode == SYNC_EXT24)) {
  
      // external clock
  
      if (digitalRead(PIN_SYNC_IN) == HIGH) { // LOW instead of HIGH because of INPUT_PULLUP?
        #ifdef DEBUG
          Serial.println("SYNC SIGNAL.");
        #endif
        if (!gSyncOn) {
          gSyncOn = true;
          if (gSyncMode == SYNC_EXT24) {
            if (gSyncCount == 0) {
              gSyncNoteOn = true;
            }
            gSyncCount++;
            if (gSyncCount == 6) {
              gSyncCount = 0;
            }
          } else {
            gSyncNoteOn = true;
          }
        } else {
          gSyncNoteOn = false;
        }
        // for SYNC_EXT, copy sync behavior to SYNC_OUT
        digitalWrite(PIN_SYNC_OUT, HIGH);

      } else {
        digitalWrite(PIN_SYNC_OUT, LOW);
        gSyncOn = false;
        gSyncNoteOn = false;
      }
  
    } else {
  
      // internal clock (SYNC_NONE, SYNC_IN)

      // has 1/16 passed ie time for new note?
      if ((gSeqTimeCurrent - gSeqTimeLast >= gSeqBase.gSeqT16) && !gSeqNoteOn)
      {

        gSyncNoteOn = true;
        
        gSeqTimeLast = gSeqTimeLast + gSeqBase.gSeqT16; // gSeqTimeLast = gSeqTimeCurrent;

      } else {
      
        gSyncNoteOn = false;
        
      } // >= gSeqBase.gSeqT16?
    }  
  
    // time to play next note

    if (gSyncNoteOn) {
              
      // NOTE ON

      // handle SYNC OUT
      if (gSyncMode == SYNC_INT) {
        digitalWrite(PIN_SYNC_OUT, HIGH);        
      }

      playNote();
      
      gSyncNoteOn = false;

      digitalWrite(PIN_LED, HIGH);

      // Sequence/Mozzi
      
      gSeqNoteIndex++;
      if (gSeqNoteIndex >= MAX_NOTES) {
        gSeqNoteIndex = 0;
      }

    }

    // after half 1/16

    if (gSeqTimeCurrent - gSeqTimeLast >= gSeqTimeGate) {
      
      gSeqNoteOn = false;
      digitalWrite(PIN_LED, LOW);

      // handle SYNC OUT
      if (gSyncMode == SYNC_INT) {
        digitalWrite(PIN_SYNC_OUT, LOW);        
      }

    }

  }

  // update envelopes

  if (gSeqPlay || gPlayMode != 0) {
    
    gMEnvKickA.update();
    gMEnvKickP.update();
    gMDOscKick.setFreq(gMSynthKick.sMFrequency + (gMSynthKick.sMEnvValueP >> gMSynthKick.sMEnvSlope)); // (setting)

    gMEnvSnareA.update();
    gMEnvSnareP.update();
    gMDOscSnare.setFreq(gMSynthSnare.sMFrequency + (gMSynthSnare.sMEnvValueP >> gMSynthSnare.sMEnvSlope)); // (setting)

    gMEnvHihatA.update();

    gMEnvClapA.update();

    gMEnvCrashA.update();

    gMEnvTomHiA.update();
    gMEnvTomHiP.update();
    gMDOscTomHi.setFreq(gMSynthTomHi.sMFrequency + (gMSynthTomHi.sMEnvValueP >> gMSynthTomHi.sMEnvSlope)); // (setting)

    if (gUIMode == UI_MODE_START) {
      gUILCD.setCursor(15, 1);
      gUILCD.print(gSeqNoteIndex, HEX);
    }

  }

  // UI
  
  UIHandle();

}



// MOZZI

AudioOutput updateAudio(){

  if (gSeqPlay || gPlayMode != 0) {
    gMSynthKick.sMEnvValueA = gMEnvKickA.next();
    gMSynthKick.sMEnvValueP = gMEnvKickP.next();
    gMSynthSnare.sMEnvValueA = gMEnvSnareA.next();
    gMSynthSnare.sMEnvValueP = gMEnvSnareP.next();
    gMSynthHihat.sMEnvValueA = gMEnvHihatA.next();
    gMSynthClap.sMEnvValueA = gMEnvClapA.next();
    gMSynthCrash.sMEnvValueA = gMEnvCrashA.next();
    gMSynthTomHi.sMEnvValueA = gMEnvTomHiA.next();
    gMSynthTomHi.sMEnvValueP = gMEnvTomHiP.next();

    return (
                    ((gMSynthKick.sMEnvValueA * gMDOscKick.next()) >> 1)
                  + ((gMSynthSnare.sMEnvValueA * (gMDOscSnare.next() + gMDOscSnareN.next())>> 2))
                  + ((gMSynthHihat.sMEnvValueA * gMDOscHihatN.next()) >> 1)
                  + ((gMSynthClap.sMEnvValueA * gMDOscClapN.next()) >> 1)
                  + ((gMSynthTomHi.sMEnvValueA * gMDOscTomHi.next()) >> 1)
                  + ((gMSynthCrash.sMEnvValueA * gMDOscCrashN.next()) >> 2)
                ) >> 8;
 
  } else {
    return 0;
  }
}



// ARDUINO

void loop()
{
  audioHook();
}
