/*
  
Project: OscPocketO - Synth
Description: Arduino Pocket Synth using the Mozzi library v2
Author: Staffan Melin, staffan.melin@oscillator.se
License: GNU General Public License v3.0
Version: 20240904
Project site: http://www.oscillator.se/arduino

Mozzi:
https://groups.google.com/g/mozzi-users/c/naotfqflOTo
https://sensorium.github.io/Mozzi/learn/porting/
https://sensorium.github.io/Mozzi/learn/configuration/

*/

//#define DEBUG

#include <MozziConfigValues.h>
#define MOZZI_ANALOG_READ_RESOLUTION 10
#define MOZZI_AUDIO_PIN_1 TIMER1_B_PIN
#define MOZZI_AUDIO_PIN_1_REGISTER OCR1B
// probably not necessary
#define MOZZI_AUDIO_PIN_2 TIMER1_A_PIN
#define MOZZI_AUDIO_PIN_2_REGISTER OCR1A
#include <Mozzi.h>

#include <Oscil.h> // oscillator template
#include <ADSR.h>
#include <mozzi_midi.h> // mtof()
//#include <LowPassFilter.h>
#include <ResonantFilter.h>
#include <tables/sin2048_int8.h>
#include <tables/triangle2048_int8.h>
#include <tables/saw2048_int8.h>
#include <tables/square_no_alias_2048_int8.h> 

// LCD
#include <LiquidCrystal.h>

// EEPROM
#include <EEPROM.h>



// MOZZI

// use #define for CONTROL_RATE, not a constant
#define MOZZI_CONTROL_RATE 128 // Hz, powers of 2 are most reliable

#define SYNTH_LEVEL 128 // ADSR max value; larger values seem to mess upp the low pass filter

// a struct of synth settings is good because then we can easily save it into EEPROM
typedef struct {
  byte gMWaveform;
  unsigned int gMAttackTime;
  unsigned int gMDecayTime;
  unsigned int gMSustainTime;
  unsigned int gMReleaseTime;
  byte gMFilterMode;
  uint8_t gMFilterCutoff;
  uint8_t gMFilterResonance;
  byte gMWaveform2;
  byte gMDetune2;
} MSynth;

MSynth gMSynth;

typedef struct {
  uint8_t gSeqGatePercent;
  byte gSeqBPM = 120; // tempo, BPM (beats per minute)
  unsigned int gSeqT16 = 125; // length in ms of 1/16
} SeqBase;

SeqBase gSeqBase;

Oscil <SAW2048_NUM_CELLS, MOZZI_AUDIO_RATE> gMOscillator(SAW2048_DATA);

#define WAVEFORM_NONE 0
#define WAVEFORM_SIN 1
#define WAVEFORM_TRI 2
#define WAVEFORM_SAW 3
#define WAVEFORM_SQUARE 4

// envelope
ADSR <MOZZI_CONTROL_RATE, MOZZI_AUDIO_RATE> gMEnvelope;

// ADSR
byte gMAttackLevel;
byte gMDecayLevel;
byte gMSustainLevel;
byte gMReleaseLevel;

byte gMGain;

LowPassFilter gMRes;

#define FILTER_MODE_FIXED 0
#define FILTER_MODE_RANDOM 1
#define FILTER_MODE_SLOW 2
#define FILTER_MODE_FAST 3
#define FILTER_MODE_POT 4

enum FilterDirection {FILTER_UP, FILTER_DOWN};
FilterDirection gMFilterDirection;

// oscillator 2
Oscil <SAW2048_NUM_CELLS, MOZZI_AUDIO_RATE> gMOscillator2(SAW2048_DATA);



// SEQUENCER

// index of current note (0-MAX_NOTES)
byte gSeqPatternIndex;
byte gSeqNoteIndex;
byte gSeqNoteEdit; // currently edited note, < MAX_NOTES
unsigned long gSeqTimeCurrent;
unsigned long gSeqTimeLast;
unsigned long gSeqTimeGate;
bool gSeqNoteOn;
bool gSeqPlay; // true if sequence is playing

byte gPlayMode; // play mode: 0 = no sound; 10-13 = note 1-3 is playing

#define MAX_PATTERNS 3
#define MAX_NOTES 16 // max # of notes in sequence
byte gSeqNotes[MAX_PATTERNS][MAX_NOTES] = 
  {
    {
      48, 48, 51, 53, 
      48, 48, 48, 48, 
      51, 48, 53, 48, 
      48, 48, 51, 53
    },
    {
      60, 63, 0, 63, 
      65, 0, 0, 63, 
      60, 63, 60, 0, 
      0, 60, 63, 60
    },
    {
      36, 0, 36, 0, 
      0, 0, 0, 0, 
      36, 0, 36, 0, 
      0, 0, 0, 0
    }
  };

#define STATE_OFF 0
#define STATE_ON 1
byte gSeqStatus[MAX_PATTERNS][MAX_NOTES] = 
  {
    {
    STATE_ON, STATE_ON, STATE_ON, STATE_ON, 
    STATE_ON, STATE_ON, STATE_ON, STATE_ON, 
    STATE_ON, STATE_ON, STATE_ON, STATE_ON, 
    STATE_ON, STATE_ON, STATE_ON, STATE_ON
    },
    {
    STATE_ON, STATE_ON, STATE_OFF, STATE_ON, 
    STATE_ON, STATE_OFF, STATE_OFF, STATE_ON, 
    STATE_ON, STATE_ON, STATE_ON, STATE_OFF, 
    STATE_OFF, STATE_ON, STATE_ON, STATE_ON
    },
    {
    STATE_ON, STATE_OFF, STATE_ON, STATE_OFF, 
    STATE_OFF, STATE_OFF, STATE_OFF, STATE_OFF, 
    STATE_ON, STATE_OFF, STATE_ON, STATE_OFF, 
    STATE_OFF, STATE_OFF, STATE_OFF, STATE_OFF
    }
  };

#define PIN_SYNC_IN 11
#define PIN_SYNC_OUT 12
bool gSyncOn = false;
bool gSyncNoteOn = false;
byte gSyncMode;
byte gSyncCount;
#define SYNC_NONE 0 // use internal sync, do not send out sync
#define SYNC_INT 1 // use internal sync, send out sync
#define SYNC_EXT 2 // use external sync, send out sync
#define SYNC_EXT24 3 // use external sync, 24ppq

#define MIDI_MAX 120
#define MIDI_MIN 0

// UI

#define PIN_LED 13

// LCD
#define PIN_LCD_D4 4
#define PIN_LCD_D5 5
#define PIN_LCD_D6 6
#define PIN_LCD_D7 7
#define PIN_LCD_RS 8
#define PIN_LCD_EN 9

// buttons, all
#define PIN_BUTTONS A0

#define PIN_POT1 A1
#define PIN_POT2 A2

// LCD
LiquidCrystal gUILCD(PIN_LCD_RS, PIN_LCD_EN, PIN_LCD_D4, PIN_LCD_D5, PIN_LCD_D6, PIN_LCD_D7);

enum UIMode {UI_MODE_START, UI_MODE_SYNC, UI_MODE_PATTERN,
  UI_MODE_EDIT, UI_MODE_STATE, UI_MODE_TEMPO, UI_MODE_GATE, UI_MODE_SHIFT, 
  UI_MODE_WAVEFORM, UI_MODE_ADSR_ATTACK, UI_MODE_ADSR_RELEASE, 
  UI_MODE_FILTER_MODE, UI_MODE_FILTER_CUTOFF, UI_MODE_FILTER_RESONANCE, 
  UI_MODE_WAVEFORM2, UI_MODE_DETUNE2, 
  UI_MODE_PLAY, UI_MODE_TOOLS};

UIMode gUIMode = UI_MODE_START;
byte gUIModeTools = 0;

enum UIButton {UI_BUTTON_NONE, UI_BUTTON_SELECT, UI_BUTTON_LEFT, UI_BUTTON_RIGHT, UI_BUTTON_UP, UI_BUTTON_DOWN};




// UTILITY

/*
  calculate length of 1/16 in ms based on bpm

  number of 1/16 per minute: bpm * 4
  number of 1/16 per second: (bpm * 4) / 60
  each 1/16 is in ms: 1000 / ((bpm * 4) / 60)
  eg: 1000 / ((120 * 4) / 60) = 1000 / (480 / 60) = 1000 / 8 = 125
*/
void setTempo(void) {
  gSeqBase.gSeqT16 = 1000 / ((gSeqBase.gSeqBPM * 4) / ((float)60));
}



void calcGate() {
  gSeqTimeGate = (gSeqBase.gSeqT16 * gSeqBase.gSeqGatePercent) / 100;        
}




void setWaveform() {
  switch (gMSynth.gMWaveform) {
    case WAVEFORM_SIN:
      gMOscillator.setTable(SIN2048_DATA);
      break;
    case WAVEFORM_TRI:
      gMOscillator.setTable(TRIANGLE2048_DATA);
      break;
    case WAVEFORM_SAW:
      gMOscillator.setTable(SAW2048_DATA);
      break;
    case WAVEFORM_SQUARE:
      gMOscillator.setTable(SQUARE_NO_ALIAS_2048_DATA);
      break;
  }  
}


void setWaveform2() {
  switch (gMSynth.gMWaveform2) {
    case WAVEFORM_SIN:
      gMOscillator2.setTable(SIN2048_DATA);
      break;
    case WAVEFORM_TRI:
      gMOscillator2.setTable(TRIANGLE2048_DATA);
      break;
    case WAVEFORM_SAW:
      gMOscillator2.setTable(SAW2048_DATA);
      break;
    case WAVEFORM_SQUARE:
      gMOscillator2.setTable(SQUARE_NO_ALIAS_2048_DATA);
      break;
  }  
}


void setADSRTimes() {
  gMEnvelope.setTimes(gMSynth.gMAttackTime, gMSynth.gMDecayTime, gMSynth.gMSustainTime, gMSynth.gMReleaseTime);
}



void playNote() {
  if (gSeqStatus[gSeqPatternIndex][gSeqNoteIndex] == STATE_ON) {
    byte noteMidi = gSeqNotes[gSeqPatternIndex][gSeqNoteIndex];
    int noteFreq = mtof(noteMidi);
    gMOscillator.setFreq(noteFreq);
    if (gMSynth.gMWaveform2 != WAVEFORM_NONE) {
      gMOscillator2.setFreq(noteFreq + gMSynth.gMDetune2); 
    }
    gMEnvelope.noteOn();
    digitalWrite(PIN_LED, HIGH);
    gSeqNoteOn = true;
  }  
}



void playNoteOn(byte aNote) {
  byte noteMidi = gSeqNotes[gSeqPatternIndex][aNote];
  int noteFreq = mtof(noteMidi);
  gMOscillator.setFreq(noteFreq);
  if (gMSynth.gMWaveform2 != WAVEFORM_NONE) {
    gMOscillator2.setFreq(noteFreq + gMSynth.gMDetune2); 
  }
//  gEnvelope.setSustainTime(100000);
  gMEnvelope.noteOn();
  digitalWrite(PIN_LED, HIGH);
}

void playNoteOff() {
//  gMEnvelope.setSustainTime(10000);
  gMEnvelope.noteOff();
  digitalWrite(PIN_LED, LOW);
}



// EEPROM size: 1024 bytes
void saveToEEPROM() {
  int aAddress = 0;

  // save synth settings
  EEPROM.put(aAddress, gMSynth);
  aAddress += sizeof(MSynth);

  // save sequencer settings
  EEPROM.put(aAddress, gSeqBase);
  aAddress += sizeof(SeqBase);

  // save note data
  for (byte p = 0; p < MAX_PATTERNS; p++) {
    for (byte n = 0; n < MAX_NOTES; n++) {
      EEPROM.write(aAddress++, gSeqNotes[p][n]);
      EEPROM.write(aAddress++, gSeqStatus[p][n]);
    }
  }

}



void loadFromEEPROM() {
  int aAddress = 0;

  // load synth settings
  EEPROM.get(aAddress, gMSynth);
  aAddress += sizeof(MSynth);

  // load sequencer settings
  EEPROM.get(aAddress, gSeqBase);
  aAddress += sizeof(SeqBase);

  // load note data  
  for (byte p = 0; p < MAX_PATTERNS; p++) {
    for (byte n = 0; n < MAX_NOTES; n++) {
      gSeqNotes[p][n] = EEPROM.read(aAddress++);
      gSeqStatus[p][n] = EEPROM.read(aAddress++);
    }
  }
  
}



// UI

void UIDraw() {
  
  gUILCD.clear();
  
  switch (gUIMode) {
    
  case UI_MODE_START:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: START SYNTH");
    gUILCD.setCursor(0, 1);
    if (gSeqPlay) {
      gUILCD.print ("Play");
    } else {
      gUILCD.print ("Stop");
    } 
    break;

  case UI_MODE_SYNC:
    gUILCD.setCursor(0, 0);
    gUILCD.print("M: SYNC");
    gUILCD.setCursor(0, 1);
    switch (gSyncMode) {
      case SYNC_NONE:
        gUILCD.print("None");
        break;
      case SYNC_INT:
        gUILCD.print("Internal");
        break;
      case SYNC_EXT:
        gUILCD.print("External");
        break;
      case SYNC_EXT24:
        gUILCD.print("Ext24");
        break;
    }
    break;

  case UI_MODE_PATTERN:
    gUILCD.setCursor(0, 0);
    gUILCD.print("M: PATTERN");
    gUILCD.setCursor(0, 1);
    gUILCD.write('0' + gSeqPatternIndex);
    break;

  case UI_MODE_STATE:
    gUILCD.setCursor(0, 0);
    gUILCD.print("M: STATE");
    for (byte i = 0; i < MAX_NOTES; i++) {
      gUILCD.setCursor(i, 1);
      switch (gSeqStatus[gSeqPatternIndex][i]) {
        case STATE_OFF:
          gUILCD.write('O');
          break;
        case STATE_ON:
          gUILCD.write('X');
          break;
      }
    }
    gUILCD.setCursor(gSeqNoteEdit, 1);
    break;

  case UI_MODE_EDIT:
  case UI_MODE_SHIFT:
    gUILCD.setCursor(0, 0);
    for (byte i = 0; i < MAX_NOTES; i++) {
      byte aValue = gSeqNotes[gSeqPatternIndex][i];
      byte aTen = aValue / 10;
      gUILCD.setCursor(i, 0);
      gUILCD.write('0' + aTen);
      gUILCD.setCursor(i, 1);
      gUILCD.write('0' +  aValue - aTen * 10);
    }
    gUILCD.setCursor(gSeqNoteEdit, 1);
    break;

  case UI_MODE_TEMPO:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: TEMPO");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gSeqBase.gSeqBPM);
    break;
    
  case UI_MODE_GATE:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: GATE");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gSeqBase.gSeqGatePercent);
    break;
    
  case UI_MODE_WAVEFORM:
    gUILCD.setCursor(0, 0);
    gUILCD.print("M: WAVEFORM");
    gUILCD.setCursor(0, 1);
    switch (gMSynth.gMWaveform) {
      case WAVEFORM_NONE:
        gUILCD.print("---");
        break;
      case WAVEFORM_SIN:
        gUILCD.print("Sin");
        break;
      case WAVEFORM_TRI:
        gUILCD.print("Tri");
        break;
      case WAVEFORM_SAW:
        gUILCD.print("Saw");
        break;
      case WAVEFORM_SQUARE:
        gUILCD.print("Square");
        break;
    }
    break;

  case UI_MODE_ADSR_ATTACK:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: ADSR ATACK");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gMSynth.gMAttackTime);
    break;
    
  case UI_MODE_ADSR_RELEASE:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: ADSR RELEASE");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gMSynth.gMReleaseTime);
    break;

  case UI_MODE_FILTER_MODE:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: FILTER MODE");
    gUILCD.setCursor(0, 1);
    switch (gMSynth.gMFilterMode) {
      case FILTER_MODE_FIXED:
        gUILCD.print("Fixed");
        break;
      case FILTER_MODE_RANDOM:
        gUILCD.print("Random");
        break;
      case FILTER_MODE_SLOW:
        gUILCD.print("Slow");
        break;
      case FILTER_MODE_FAST:
        gUILCD.print("Fast");
        break;
      case FILTER_MODE_POT:
        gUILCD.print("Pots");
        break;
    }
    break;

  case UI_MODE_FILTER_CUTOFF:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: FILTER CUTOFF");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gMSynth.gMFilterCutoff);
    break;

  case UI_MODE_FILTER_RESONANCE:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: FILTER RES");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gMSynth.gMFilterResonance);
    break;

  case UI_MODE_WAVEFORM2:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: WAVEFORM 2");
    gUILCD.setCursor(0, 1);
    switch (gMSynth.gMWaveform2) {
      case WAVEFORM_NONE:
        gUILCD.print("---");
        break;
      case WAVEFORM_SIN:
        gUILCD.print("Sin");
        break;
      case WAVEFORM_TRI:
        gUILCD.print("Tri");
        break;
      case WAVEFORM_SAW:
        gUILCD.print("Saw");
        break;
      case WAVEFORM_SQUARE:
        gUILCD.print("Square");
        break;
    }
    break;

  case UI_MODE_DETUNE2:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: DETUNE 2");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gMSynth.gMDetune2);
    break;

  case UI_MODE_PLAY:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: PLAY (LEFT)");
    gUILCD.setCursor(0, 1);
    gUILCD.print(gSeqNotes[gSeqPatternIndex][0]);
    gUILCD.setCursor(4, 1);
    gUILCD.print(gSeqNotes[gSeqPatternIndex][1]);
    gUILCD.setCursor(8, 1);
    gUILCD.print(gSeqNotes[gSeqPatternIndex][2]);
    gUILCD.setCursor(12, 1);
    gUILCD.print(gSeqNotes[gSeqPatternIndex][3]);
    break;

  case UI_MODE_TOOLS:
    gUILCD.setCursor(0, 0);
    gUILCD.print ("M: TOOLS");
    gUILCD.setCursor(0, 1);
    gUILCD.print("SLRBC");
    gUILCD.setCursor(gUIModeTools, 1);
    break;

  }

}

UIButton UIHandle() {
  uint16_t aUIButtonValue;
  UIButton aUIButton = UI_BUTTON_NONE;

  static unsigned long aUIButtonPreviousTime = millis();

  aUIButtonValue = mozziAnalogRead(PIN_BUTTONS);

  #ifdef DEBUG
//    Serial.println(aUIButtonValue);
  #endif

  // is any button pressed?
  if (aUIButtonValue < 900)
  {
    // check which one
    if (aUIButtonValue < 50)
    {
      aUIButton = UI_BUTTON_RIGHT;
    } else if (aUIButtonValue < 150) {
      aUIButton = UI_BUTTON_UP;
    } else if (aUIButtonValue < 300) {
      aUIButton = UI_BUTTON_DOWN;
    } else if (aUIButtonValue < 500) {
      aUIButton = UI_BUTTON_LEFT;
    } else if (aUIButtonValue < 700) {
      aUIButton = UI_BUTTON_SELECT;
    } else {
      aUIButton = UI_BUTTON_NONE;
    }

    // rightmost button is RST = reset = resets Arduino

    // debounce buttons
    if (aUIButton != UI_BUTTON_NONE && (millis() - aUIButtonPreviousTime > 200)) {
      aUIButtonPreviousTime = millis();
      
      switch (gUIMode) {
        
      case UI_MODE_START:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_SYNC;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (!gSeqPlay) {
            gSeqNoteIndex = 0;
            gSeqTimeCurrent = millis();
            gSeqTimeLast = gSeqTimeCurrent;
            gSeqNoteOn = false;
            //playNote();
            gSeqPlay = true;
            gSyncNoteOn = false;
            gSyncOn = false;
            gSyncCount = 0;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqPlay) {
            gSeqPlay = false;
            gMEnvelope.noteOff();
            digitalWrite(PIN_LED, LOW);            
            gSeqNoteOn = false;
          }
        } 
        break;
        
      case UI_MODE_SYNC:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_PATTERN;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSyncMode < SYNC_EXT24)
          {
            gSyncMode++;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSyncMode > 0)
          {
            gSyncMode--;
            gSeqTimeLast = gSeqTimeCurrent; // going back to SYNC_INT (or SYNC_NONE), make sure gSeqTimeLast is updated
          }
        }
        break;

      case UI_MODE_PATTERN:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_EDIT;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqPatternIndex < (MAX_PATTERNS - 1))
          {
            gSeqPatternIndex++;
          } else {
            gSeqPatternIndex = 0;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqPatternIndex > 0)
          {
            gSeqPatternIndex--;
          } else {
            gSeqPatternIndex = (MAX_PATTERNS - 1);            
          }
        }
        break;

      case UI_MODE_EDIT:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_STATE;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqNotes[gSeqPatternIndex][gSeqNoteEdit] < MIDI_MAX)
          {
            gSeqNotes[gSeqPatternIndex][gSeqNoteEdit]++;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqNotes[gSeqPatternIndex][gSeqNoteEdit] > 0)
          {
            gSeqNotes[gSeqPatternIndex][gSeqNoteEdit]--;
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gSeqNoteEdit > 0) {
            gSeqNoteEdit--;
          }
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gSeqNoteEdit < MAX_NOTES - 1) {
            gSeqNoteEdit++;
          }
        }
        break;

      case UI_MODE_STATE:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_TEMPO;
        } else if (aUIButton == UI_BUTTON_UP) {
          gSeqStatus[gSeqPatternIndex][gSeqNoteEdit] = STATE_ON;
        } else if (aUIButton == UI_BUTTON_DOWN) {
          gSeqStatus[gSeqPatternIndex][gSeqNoteEdit] = STATE_OFF;
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gSeqNoteEdit > 0) {
            gSeqNoteEdit--;
          }
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gSeqNoteEdit < MAX_NOTES - 1) {
            gSeqNoteEdit++;
          }
        }
        break;

      case UI_MODE_TEMPO:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_GATE;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqBase.gSeqBPM < 999)
          {
            gSeqBase.gSeqBPM++;
            setTempo();
            calcGate();
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqBase.gSeqBPM > 10)
          {
            gSeqBase.gSeqBPM--;
            setTempo();
            calcGate();
          }
        }
        break;

      case UI_MODE_GATE:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_SHIFT;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqBase.gSeqGatePercent < 99)
          {
            gSeqBase.gSeqGatePercent++;
            calcGate();
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqBase.gSeqGatePercent > 0)
          {
            gSeqBase.gSeqGatePercent--;
            calcGate();
          }
        }

        break;

      case UI_MODE_SHIFT:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_WAVEFORM;
        } else if (aUIButton == UI_BUTTON_UP) {
          for (byte i = 0; i < MAX_NOTES; i++) {
            if (gSeqNotes[gSeqPatternIndex][i] < MIDI_MAX) {
              gSeqNotes[gSeqPatternIndex][i]++;
            }
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          for (byte i = 0; i < MAX_NOTES; i++) {
            if (gSeqNotes[gSeqPatternIndex][i] > MIDI_MIN) {
              gSeqNotes[gSeqPatternIndex][i]--;
            }
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          byte savedNote = gSeqNotes[gSeqPatternIndex][0];
          for (byte i = 0; i < MAX_NOTES - 1; i++) {
            gSeqNotes[gSeqPatternIndex][i] = gSeqNotes[gSeqPatternIndex][i+1];
          }
          gSeqNotes[gSeqPatternIndex][MAX_NOTES-1] = savedNote;
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          byte savedNote = gSeqNotes[gSeqPatternIndex][MAX_NOTES-1];
          for (byte i = MAX_NOTES - 1; i > 0; i--) {
            gSeqNotes[gSeqPatternIndex][i] = gSeqNotes[gSeqPatternIndex][i-1];
          }
          gSeqNotes[gSeqPatternIndex][0] = savedNote;
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gSeqPatternIndex > 0) {
            gSeqPatternIndex--;
          }
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gSeqPatternIndex < MAX_PATTERNS - 1) {
            gSeqPatternIndex++;
          }
        } 
        break;

      case UI_MODE_WAVEFORM:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_ADSR_ATTACK;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynth.gMWaveform < 4)
          {
            gMSynth.gMWaveform++;
            setWaveform();
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynth.gMWaveform > 1) // WAVEFORM_NONE not available for oscillator 1
          {
            gMSynth.gMWaveform--;
            setWaveform();
          }
        }
        break;

      case UI_MODE_ADSR_ATTACK:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_ADSR_RELEASE;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynth.gMAttackTime < 5000)
          {
            gMSynth.gMAttackTime++;
            setADSRTimes();
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynth.gMAttackTime > 0)
          {
            gMSynth.gMAttackTime--;
            setADSRTimes();
          }
        }
        break;

      case UI_MODE_ADSR_RELEASE:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_FILTER_MODE;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynth.gMReleaseTime < 5000)
          {
            gMSynth.gMReleaseTime += 10;
            setADSRTimes();
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynth.gMReleaseTime > 10)
          {
            gMSynth.gMReleaseTime -= 10;
            setADSRTimes();
          }
        }
        break;

      case UI_MODE_FILTER_MODE:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_FILTER_CUTOFF;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynth.gMFilterMode < 4)
          {
            gMSynth.gMFilterMode++;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynth.gMFilterMode > 0)
          {
            gMSynth.gMFilterMode--;
          }
        }
        break;
        
      case UI_MODE_FILTER_CUTOFF:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_FILTER_RESONANCE;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynth.gMFilterCutoff < 255)
          {
            gMSynth.gMFilterCutoff++;
            gMRes.setCutoffFreq(gMSynth.gMFilterCutoff);
            gMRes.setResonance(gMSynth.gMFilterResonance);
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynth.gMFilterCutoff > 0)
          {
            gMSynth.gMFilterCutoff--;
            gMRes.setCutoffFreq(gMSynth.gMFilterCutoff);
            gMRes.setResonance(gMSynth.gMFilterResonance);
          }
        }
        break;
        
      case UI_MODE_FILTER_RESONANCE:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_WAVEFORM2;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynth.gMFilterResonance < 255)
          {
            gMSynth.gMFilterResonance++;
            gMRes.setCutoffFreq(gMSynth.gMFilterCutoff);
            gMRes.setResonance(gMSynth.gMFilterResonance);
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynth.gMFilterResonance > 0)
          {
            gMSynth.gMFilterResonance--;
            gMRes.setCutoffFreq(gMSynth.gMFilterCutoff);
            gMRes.setResonance(gMSynth.gMFilterResonance);
          }
        }
        break;

      case UI_MODE_WAVEFORM2:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_DETUNE2;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynth.gMWaveform2 < 4)
          {
            gMSynth.gMWaveform2++;
            setWaveform2();
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynth.gMWaveform2 > 0)
          {
            gMSynth.gMWaveform2--;
            setWaveform2();
          }
        }
        break;

      case UI_MODE_DETUNE2:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_PLAY;
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gMSynth.gMDetune2 < 255)
          {
            gMSynth.gMDetune2++;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gMSynth.gMDetune2 > 0)
          {
            gMSynth.gMDetune2--;
          }
        }
        break;

      case UI_MODE_PLAY:
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_TOOLS;
          gPlayMode = 0;
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gPlayMode == 0) {
            // prep
            gSeqPlay = false;
            gMEnvelope.noteOff();
            digitalWrite(PIN_LED, LOW);            
            gSeqNoteOn = false;
            gSyncMode = SYNC_NONE;
            gPlayMode = 1;
          } else if (gPlayMode != 10) {
            playNoteOn(0);
            gPlayMode = 10;
          }
        } else if (aUIButton == UI_BUTTON_UP) {
          if (gPlayMode != 11) {
            playNoteOn(1);
            gPlayMode = 11;
          }
        } else if (aUIButton == UI_BUTTON_DOWN) {
          if (gPlayMode != 12) {
            playNoteOn(2);
            gPlayMode = 12;
          }
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gPlayMode != 13) {
            playNoteOn(3);
            gPlayMode = 13;
          }
        }
        break;

      case UI_MODE_TOOLS:
        // Save/Load/Random/Bassline/Copytopatternover
        if (aUIButton == UI_BUTTON_SELECT) {
          gUIMode = UI_MODE_START;
          // because the LCD messes up from time to time
          gUILCD.begin(16, 2);
          gUILCD.cursor();
        } else if (aUIButton == UI_BUTTON_RIGHT) {
          if (gUIModeTools < 4)
          {
            gUIModeTools++;
          }
        } else if (aUIButton == UI_BUTTON_LEFT) {
          if (gUIModeTools > 0)
          {
            gUIModeTools--;
          }
        } else if (aUIButton == UI_BUTTON_UP) {
          switch (gUIModeTools) {
            case 0: // Save
              // stop playing (because this takes time)
              gSeqPlay = false;
              gMEnvelope.noteOff();
              digitalWrite(PIN_LED, LOW);            
              gSeqNoteOn = false;

              saveToEEPROM();
              break;
            case 1: // Load
              // stop playing (because this takes time)
              gSeqPlay = false;
              gMEnvelope.noteOff();
              digitalWrite(PIN_LED, LOW);            
              gSeqNoteOn = false;

              loadFromEEPROM();
              break;
            case 2: // Random
              {
                byte aNote = gSeqNotes[gSeqPatternIndex][gSeqNoteEdit];
                if (aNote < 12) {
                  aNote = aNote + 12;
                } else if (aNote > MIDI_MAX - 12) {
                  aNote = aNote - 12;
                }
                for (byte n = 0; n < MAX_NOTES; n++) {
                  gSeqNotes[gSeqPatternIndex][n] = rand()%12 + (aNote - 6);
                  if (rand()%100 > 16) {
                    gSeqStatus[gSeqPatternIndex][n] = STATE_ON;
                  } else {
                    gSeqStatus[gSeqPatternIndex][n] = STATE_OFF;
                  }
                }
                break;
              }
            case 3: // Bassline
              {
                byte aNote = gSeqNotes[gSeqPatternIndex][gSeqNoteEdit];
                for (byte n = 0; n < MAX_NOTES; n++) {
                  gSeqNotes[gSeqPatternIndex][n] = aNote;
                  gSeqStatus[gSeqPatternIndex][n] = STATE_ON;
                }
                break;
              }
            case 4: // Copy
              {
                // assume MAX_PATTERNS > 1
                // copy to next larger pattern
                byte aTo = gSeqPatternIndex + 1;
                if (aTo == MAX_PATTERNS) {
                  aTo = 0;
                }
                for (byte n = 0; n < MAX_NOTES; n++) {
                  gSeqNotes[aTo][n] = gSeqNotes[gSeqPatternIndex][n];
                  gSeqStatus[aTo][n] = gSeqStatus[gSeqPatternIndex][n];
                }
                break;
              }
          }
        }
        break;

      } // switch (gUIMode)

      UIDraw();

    } // debounce
  }  // if (gUIButton < 900)

  return (aUIButton);
}



// SETUP

void setup()
{
  #ifdef DEBUG
    Serial.begin(115200);
  #endif

  setTempo();

  // Arduino
  
  pinMode(PIN_LED, OUTPUT);
  pinMode(PIN_SYNC_IN, INPUT_PULLUP);
  pinMode(PIN_SYNC_OUT, OUTPUT);

  pinMode(PIN_POT1, INPUT);
  pinMode(PIN_POT2, INPUT);  

  gUILCD.begin(16, 2);
  gUILCD.cursor();
  gUILCD.clear();



  // MOZZI
  
  gMSynth.gMWaveform = WAVEFORM_SAW;

  // MOZZI: ADSR
  gMAttackLevel = SYNTH_LEVEL;
  gMDecayLevel = SYNTH_LEVEL;
  gMSustainLevel = SYNTH_LEVEL;
  gMReleaseLevel = 0;
  gMSynth.gMAttackTime = 0;
  gMSynth.gMDecayTime = 0;
  gMSynth.gMSustainTime = 10000;
  gMSynth.gMReleaseTime = (gSeqBase.gSeqT16 >> 1);

  gMEnvelope.setLevels(gMAttackLevel, gMDecayLevel, gMSustainLevel, gMReleaseLevel);
  // gMEnvelope.setTimes(gMSynth.gMAttackTime,gMDecayTime, gMSustainTime, gMSynth.gMReleaseTime);
  setADSRTimes();
  
  gMGain = 0;
  gMSynth.gMWaveform2 = WAVEFORM_NONE;
  gMSynth.gMDetune2 = 0;

  // MOZZI: filter

  gMSynth.gMFilterMode = FILTER_MODE_FIXED;
  gMSynth.gMFilterCutoff = 100;
  gMSynth.gMFilterResonance = 100;
  gMFilterDirection = FILTER_UP;
  gMRes.setCutoffFreq(gMSynth.gMFilterCutoff);
  gMRes.setResonance(gMSynth.gMFilterResonance);



  // SEQUENCER
  // note lengths
  // see setTempo() function
  gSeqBase.gSeqBPM = 120; // tempo, BPM (beats per minute)
  gSeqBase.gSeqT16 = 125; // length in ms of 1/16
  gSeqBase.gSeqGatePercent = 50;
  
  gSeqPatternIndex = 0;
  gSeqNoteIndex = 0;
  gSeqNoteEdit = 0; // currently edited note, < MAX_NOTES
  gSeqTimeCurrent = millis();
  gSeqTimeLast = gSeqTimeCurrent;
  gSeqTimeGate = (gSeqBase.gSeqT16 >> 1);
  gSeqNoteOn = false;
  gSeqPlay = false;

  gSyncMode = SYNC_NONE;
  gSyncCount = 0;

  gPlayMode = 0;

  UIDraw();

  startMozzi(MOZZI_CONTROL_RATE);

}



// MOZZI: put changing controls in here

void updateControl() {

  if (gSeqPlay) {

    gSeqTimeCurrent = millis();

    if ((gSyncMode == SYNC_EXT) || (gSyncMode == SYNC_EXT24)) {
  
      // external clock
  
      if (digitalRead(PIN_SYNC_IN) == HIGH) { // LOW instead of HIGH because of INPUT_PULLUP?
        if (!gSyncOn) {
          gSyncOn = true;
          if (gSyncMode == SYNC_EXT24) {
            if (gSyncCount == 0) {
              gSyncNoteOn = true;
            }
            gSyncCount++;
            if (gSyncCount == 6) {
              gSyncCount = 0;
            }
          } else {
            gSyncNoteOn = true;
          }
        } else {
          gSyncNoteOn = false;
        }
        // for SYNC_EXT, copy sync behavior to SYNC_OUT
        digitalWrite(PIN_SYNC_OUT, HIGH);
      } else {
        digitalWrite(PIN_SYNC_OUT, LOW);
        gSyncOn = false;
        gSyncNoteOn = false;
      }
  
    } else {
  
      // internal clock (SYNC_NONE, SYNC_IN)

      // has 1/16 passed ie time for new note?
      if ((gSeqTimeCurrent - gSeqTimeLast >= gSeqBase.gSeqT16) && !gSeqNoteOn)
      {

        gSyncNoteOn = true;
        
        gSeqTimeLast = gSeqTimeLast + gSeqBase.gSeqT16; // gSeqTimeLast = gSeqTimeCurrent;

      } else {
      
        gSyncNoteOn = false;
        
      } // >= gSeqBase.gSeqT16?
    }  
  
    // time to play next note

    if (gSyncNoteOn) {
          
      // NOTE ON

      playNote();
      
      gSyncNoteOn = false;

      // handle SYNC OUT
      if (gSyncMode == SYNC_INT) {
        digitalWrite(PIN_SYNC_OUT, HIGH);        
      }

      // handle filter
      
      switch (gMSynth.gMFilterMode) {
        case FILTER_MODE_FIXED:
          // we don't have to do anything as the cutoff and resonance is already set by the UI
          break;
        case FILTER_MODE_RANDOM:
          gMRes.setCutoffFreq(rand()%gMSynth.gMFilterCutoff);
          gMRes.setResonance(gMSynth.gMFilterResonance);
          break;
        case FILTER_MODE_SLOW:
          // simple triangle modulation of filter cutoff
          if (gMFilterDirection == FILTER_UP) {
            gMSynth.gMFilterCutoff++;
            if (gMSynth.gMFilterCutoff == 255) {
              gMFilterDirection = FILTER_DOWN;
            }
          } else {
            gMSynth.gMFilterCutoff--;
            if (gMSynth.gMFilterCutoff < 11) {
              gMFilterDirection = FILTER_UP;
            }
          }
          gMRes.setCutoffFreq(gMSynth.gMFilterCutoff);
          gMRes.setResonance(gMSynth.gMFilterResonance);
          break;
        case FILTER_MODE_FAST:
          // simple triangle modulation of filter cutoff
          if (gMFilterDirection == FILTER_UP) {
            gMSynth.gMFilterCutoff = gMSynth.gMFilterCutoff + 16;
            if (gMSynth.gMFilterCutoff > 239) {
              gMFilterDirection = FILTER_DOWN;
            }
          } else {
            gMSynth.gMFilterCutoff = gMSynth.gMFilterCutoff - 16;
            if (gMSynth.gMFilterCutoff < 32) {
              gMFilterDirection = FILTER_UP;
            }
          }
          gMRes.setCutoffFreq(gMSynth.gMFilterCutoff);
          gMRes.setResonance(gMSynth.gMFilterResonance);
          break;
        case FILTER_MODE_POT:
          {
            // read pot's on A1 and A2
            int aPot1 = mozziAnalogRead(PIN_POT1);
            int aPot2 = mozziAnalogRead(PIN_POT2);
            
            gMSynth.gMFilterCutoff = aPot1 >> 2;
            gMSynth.gMFilterResonance = aPot2 >> 2;

            gMRes.setCutoffFreq(gMSynth.gMFilterCutoff);
            gMRes.setResonance(gMSynth.gMFilterResonance);
          }
          break;
      } // switch (gFilterMode)


      // Sequence/Mozzi
      
      gSeqNoteIndex++;
      if (gSeqNoteIndex >= MAX_NOTES) {
        gSeqNoteIndex = 0;
      }


    }

    // handle gate and note off

    if ((gSeqTimeCurrent - gSeqTimeLast >= gSeqTimeGate) && gSeqNoteOn) {
    
      // NOTE OFF
      gMEnvelope.noteOff();
      digitalWrite(PIN_LED, LOW);
      
      gSeqNoteOn = false;

      // handle SYNC OUT
      if (gSyncMode == SYNC_INT) {
        digitalWrite(PIN_SYNC_OUT, LOW);        
      }

    }

    // update envelope
  
    gMEnvelope.update();
//    gMGain = gMEnvelope.next();

    if (gUIMode == UI_MODE_START) {
      gUILCD.setCursor(15, 1);
      gUILCD.print(gSeqNoteIndex, HEX);
    }
        

  } else { // if (gSeqPlay)
    if (gPlayMode > 0) {
      gMEnvelope.update();
//      gMGain = gMEnvelope.next();
    } else {
      gMGain = 0;
    }
    
  }

  // UI
  
  UIButton aAction = UIHandle();

  if (gUIMode == UI_MODE_PLAY) {
    if (aAction == UI_BUTTON_NONE) {
      if (gPlayMode >= 10) { 
        gPlayMode = 1;
        playNoteOff();
      }
    }
  }
}



// MOZZI

AudioOutput updateAudio(){

  if (gSeqPlay || gPlayMode > 0) {
    gMGain = gMEnvelope.next();  
  } else {
    gMGain = 0;
  }

  if (gMSynth.gMWaveform2 == WAVEFORM_NONE) {
    return gMRes.next((gMGain * gMOscillator.next()) >>8);
  } else {
    return gMRes.next((gMGain * ((gMOscillator.next() >> 1) + (gMOscillator2.next() >> 1))) >>8);
  }
}



// ARDUINO

void loop(){
  audioHook();
}
